==> ws-0.2.6.tgz: package/lib/BufferPool.js <==
/*!
 * ws: a node.js websocket client
 * Copyright(c) 2011 Einar Otto Stangvik <einaros@gmail.com>
 * MIT Licensed
 */

var util = require('util');

function BufferPool(initialSize, growStrategy, shrinkStrategy) {
  if (typeof initialSize === 'function') {
    shrinkStrategy = growStrategy;
    growStrategy = initialSize;
    initialSize = 0;
  } 
  else if (typeof initialSize === 'undefined') {
    initialSize = 0;
  }
  this._growStrategy = (growStrategy || function(db, size) {
    return db.used + size;
  }).bind(null, this);
  this._shrinkStrategy = (shrinkStrategy || function(db) {
    return initialSize;
  }).bind(null, this);
  this._buffer = new Buffer(initialSize);
  this._offset = 0;
  this._used = 0;
  this.__defineGetter__('size', function(){
    return this._buffer.length;     
  });
  this.__defineGetter__('used', function(){
    return this._used;     
  });
}

BufferPool.prototype.get = function(length) {
  if (this._offset + length > this._buffer.length) {
    // detaches previous buffer, activates a new area
    // may cause leak-like effects, although not actually to be considered leaks
    // ideally the allocation should check a map of vacant slots, so that detached
    // buffers still are useful
    var newBuf = new Buffer(this._growStrategy(length));
    this._buffer = newBuf;
    this._offset = 0;
  }
  this._used += length;
  var buf = this._buffer.slice(this._offset, this._offset + length);
  this._offset += length;
  return buf;
}

BufferPool.prototype.reset = function() {
  this._buffer = new Buffer(this._shrinkStrategy());
  this._offset = 0;
  this._used = 0;
}

module.exports = BufferPool;

==> ws-0.2.9.tgz: package/lib/BufferPool.js <==
/*!
 * ws: a node.js websocket client
 * Copyright(c) 2011 Einar Otto Stangvik <einaros@gmail.com>
 * MIT Licensed
 */

var util = require('util');

function BufferPool(initialSize, growStrategy, shrinkStrategy) {
  if (typeof initialSize === 'function') {
    shrinkStrategy = growStrategy;
    growStrategy = initialSize;
    initialSize = 0;
  }
  else if (typeof initialSize === 'undefined') {
    initialSize = 0;
  }
  this._growStrategy = (growStrategy || function(db, size) {
    return db.used + size;
  }).bind(null, this);
  this._shrinkStrategy = (shrinkStrategy || function(db) {
    return initialSize;
  }).bind(null, this);
  this._buffer = new Buffer(initialSize);
  this._offset = 0;
  this._used = 0;
  this.__defineGetter__('size', function(){
    return this._buffer == null ? 0 : this._buffer.length;
  });
  this.__defineGetter__('used', function(){
    return this._used;
  });
}

BufferPool.prototype.get = function(length) {
  if (this._buffer == null || this._offset + length > this._buffer.length) {
    // detaches previous buffer, activates a new area
    // may cause leak-like effects, although not actually to be considered leaks
    // ideally the allocation should check a map of vacant slots, so that detached
    // buffers still are useful
    var newBuf = new Buffer(this._growStrategy(length));
    this._buffer = newBuf;
    this._offset = 0;
  }
  this._used += length;
  var buf = this._buffer.slice(this._offset, this._offset + length);
  this._offset += length;
  return buf;
}

BufferPool.prototype.reset = function() {
  var len = this._shrinkStrategy();
  this._buffer = len ? new Buffer(this._shrinkStrategy()) : null;
  this._offset = 0;
  this._used = 0;
}

module.exports = BufferPool;
